package core

import (
	"fmt"

	"git.akyoto.dev/cli/q/src/build/asm"
	"git.akyoto.dev/cli/q/src/build/config"
	"git.akyoto.dev/cli/q/src/build/cpu"
)

// SaveRegister attempts to move a variable occupying this register to another register.
func (f *Function) SaveRegister(register cpu.Register) {
	var variable *Variable

	for _, v := range f.variables {
		if v.Register == register {
			variable = v
			break
		}
	}

	if variable == nil || variable.Alive == 0 {
		return
	}

	newRegister := f.cpu.MustFindFree(f.cpu.General)
	f.cpu.Reserve(newRegister)
	f.cpu.Use(newRegister)

	if config.Comments {
		f.assembler.Comment(fmt.Sprintf("save %s to %s", register, newRegister))
	}

	f.assembler.RegisterRegister(asm.MOVE, newRegister, register)
	f.cpu.Free(register)
	variable.Register = newRegister
}
