package core

import (
	"git.akyoto.dev/cli/q/src/build/ast"
	"git.akyoto.dev/cli/q/src/build/cpu"
	"git.akyoto.dev/cli/q/src/build/expression"
	"git.akyoto.dev/cli/q/src/build/token"
)

// UsesRegister returns true if evaluating the expression would write or read the given register.
func (f *Function) UsesRegister(expr *expression.Expression, register cpu.Register) bool {
	if expr.IsLeaf() {
		if expr.Token.Kind == token.Number {
			return false
		}

		name := expr.Token.Text()
		variable := f.variables[name]
		return register == variable.Register
	}

	if ast.IsFunctionCall(expr) {
		if register == f.cpu.Output[0] {
			return true
		}

		for i, parameter := range expr.Children[1:] {
			if register == f.cpu.Input[i] {
				return true
			}

			if f.UsesRegister(parameter, register) {
				return true
			}
		}

		return false
	}

	for _, child := range expr.Children {
		if f.UsesRegister(child, register) {
			return true
		}
	}

	return false
}
